""" Tests for WorkflowRequestRestService """

# pylint: disable=E0401, W0621

import datetime
import http
from unittest.mock import MagicMock

import pytest
from pyramid.testing import DummyRequest
from workflow.server import WorkflowRequestRestService

from workspaces.workflow.schema import Workflow, WorkflowRequest
from workspaces.workflow.services.workflow_info import WorkflowInfo
from workspaces.workflow.services.workflow_service import WorkflowService


@pytest.fixture
def workflow() -> Workflow:
    """
    We'll need a null workflow.

    :return: minimally-initialized null workflow
    """
    wf = Workflow()
    wf.workflow_name = "null"
    wf.templates = []
    return wf


@pytest.fixture
def pyramid_request(
    mock_workflow_info: WorkflowInfo, mock_workflow_service: WorkflowService
) -> DummyRequest:
    """
    We'll need a pyramid request.

    :param mock_workflow_info: mocked WorkflowInfo for creating request
    :param mock_workflow_service: mocked WorkflowService for creating request workflows

    :return: a pyramid request containing what we need for testing
    """
    request = DummyRequest()
    request.info = mock_workflow_info
    request.workflows = mock_workflow_service
    return request


@pytest.fixture
def workflow_request(pyramid_request: DummyRequest, workflow: Workflow) -> DummyRequest:
    """
    We'll need a workflow request.

    :param pyramid_request: initialized pyramid request using the supplied workflow
    :param workflow: workflow that will be the request's context

    :return: handy pyramid request to use in tests
    """
    pyramid_request.context = workflow
    pyramid_request.json_body = workflow.__json__()
    return pyramid_request


@pytest.fixture
def workflow_request_request(pyramid_request: DummyRequest) -> DummyRequest:
    """
    This is called a request_request because we're making a WorkflowRequest, and we're sticking it
    inside a Pyramid Request. Hence, WorkflowRequestRequest. This sucks, and we know it.

    :param pyramid_request: partially-initialized pyramid request
    :return: pyramid request containing a workflow request (the horror! the horror!)
    """
    wf_request = WorkflowRequest()
    wf_request.name = "null"
    wf_request.argument = '["-g"]'
    wf_request.state = "Unsubmitted"
    wf_request.files = []
    wf_request.results_dir = ""
    wf_request.created_at = datetime.datetime.now()
    wf_request.updated_at = datetime.datetime.now()
    pyramid_request.context = wf_request
    pyramid_request.json_body = wf_request.__json__()
    return pyramid_request


def test_create_workflow_request(workflow_request: DummyRequest):
    """
    Confirm that WorkflowRequestRestService can create a valid WorkflowRequest.

    :param workflow_request: pyramid request for use in making workflow request
    :return:
    """
    service = WorkflowRequestRestService(workflow_request)
    service.create_workflow_request()
    assert workflow_request.info.create_workflow_request.call_count == 1


def test_submit_workflow_request(workflow_request_request: DummyRequest):
    """
    Confirm that WorkflowRequestRestService can submit a pyramid request
    containing a valid WorkflowRequest

    :param workflow_request_request: mocked pyramid request
    for use in submitting workflow request

    :return:
    """
    service = WorkflowRequestRestService(workflow_request_request)
    service.submit_workflow()
    assert workflow_request_request.workflows.execute.call_count == 1


def test_create_and_submit_wf_request(workflow_request: DummyRequest):
    """
    Confirm that WorkflowRequestRestService can create and submit a WorkflowRequest
    in one swell foop.

    :param workflow_request: pyramid request
    :return:
    """
    service = WorkflowRequestRestService(workflow_request)
    service.create_and_submit()
    assert workflow_request.info.create_workflow_request.call_count == 1
    assert workflow_request.workflows.execute.call_count == 1


def test_send_carta_url_to_aat(workflow_request_request: DummyRequest):
    # Init dummy request values
    matchdict_savepoint = workflow_request_request.matchdict
    body_savepoint = workflow_request_request.json_body
    workflow_request_request.matchdict["request_id"] = -1
    workflow_request_request.json_body = {"carta_url": "http://fake.carta/url"}

    # Mock out WorkflowService method call side effect
    workflow_request_request.workflows.send_carta_url_to_aat = MagicMock()
    response = WorkflowRequestRestService(workflow_request_request).send_carta_url_to_aat()

    workflow_request_request.workflows.send_carta_url_to_aat.assert_called_with(
        -1, "http://fake.carta/url"
    )
    assert response.status_code == http.HTTPStatus.OK

    # Reset dummy request properties to their initial values
    workflow_request_request.matchdict = matchdict_savepoint
    workflow_request_request.json_body = body_savepoint
